#include "SDLV110.h"
#include <cstdio>
#include <cmath>



SSME::SSME(VESSEL* pVessel)
: BASICENGINE(pVessel)
{
	thMe = NULL;
	fTimer1 = 0.0;
	fTimer2 = 0.0;
	fIsGood = false;
	mode = MODE_IDLE;
}


SSME::~SSME()
{

}

void SSME::Create(VECTOR3 &pos, VECTOR3& dir0, PROPELLANT_HANDLE phSource)
{
	if(!IsRealized())
	{
		BASICENGINE::Create(pos, dir0, phSource);
		thMe = pParent->CreateThruster(pos, dir0, SSME_THRUST0 * 0.002, 
					phSource, SSME_ISP0 * 0.002, SSME_ISPSL * 0.002);
		SURFHANDLE shSSMETex = oapiRegisterExhaustTexture("exhaust_atsme");
		pParent->AddExhaust(thMe, 40.0, 2.0, shSSMETex);
		SetMaximumNumberOfIgniters(2);
		this->SetNeutralThrustVector(dir0);
		SetTVCLimits(8.5 * RAD, 20.0 * RAD);
		fTimer1 = 0.0;
		fTimer2 = 0.0;
		fIsGood = false;
		fThrustCmd2 = 0.0;
		mode = MODE_IDLE;
		Realize();
	}
}



void SSME::Destroy()
{
	if(IsRealized())
	{
		pParent->DelThruster(thMe);
		thMe = NULL;
		fIsGood = false;
		Unrealize();
	}
}

void SSME::Ignite()
{
	if(IsRealized())
	{
		if(mode == MODE_IDLE)
		{
			mode = MODE_START;
		}
	}
}

bool SSME::IsGood() const
{
	if(IsRealized())
	{
		return fIsGood;
	} else
	{
		return false;
	}
}





void SSME::CalcEngineParameters(double fSimT, double fDeltaT)
{
	if(IsRealized())
	{	
		pParent->SetThrusterDir(thMe, GetThrustVector());

		
		double fThrustLevel = pParent->GetThrusterLevel(thMe);


		//fThrustLevel *= (1.0 + fDeltaT * (rand()%200 - 100)/10000.0);

		double fInertiaFactor = 1.0;
		if(!IsIgnited())
			fInertiaFactor = 0.8;

		/*
		if(GetChamberPressure() > 180.0 && !fIsGood && fTimer1 < 0.6) 
		{
			fTimer1 += fDeltaT;
			if(fTimer1 >= 0.6)
			{
				fIsGood = true;
				fTimer1 = 0.0;
			}
		}
		*/
		/*
		if(fabs(fThrustLevel - fThrustCmd) > 0.05 && fIsGood)
		{
			fTimer1 += fDeltaT;
			if(fTimer1 > 1.5)
			{
				fTimer1 = 1.5;
				fIsGood = false;
			}
		} else if(fIsGood)
		{
			fTimer1 = 0.0;
		}
		*/

		if(fThrustLevel <= fThrustCmd2 - 0.7 * fDeltaT * fInertiaFactor)
		{
			fThrustLevel += 0.7 * fDeltaT * fInertiaFactor;
		} 
		else if(fThrustLevel >= fThrustCmd2 + 0.7 * fDeltaT * fInertiaFactor)
		{
			fThrustLevel -= 0.7 * fDeltaT * fInertiaFactor;
		}
		else
		{
			fThrustLevel = fThrustCmd2;
		}

		if(IsIgnited())
		{
			pParent->SetThrusterIsp(thMe, SSME_ISP0, SSME_ISPSL);
			pParent->SetThrusterMax0(thMe, SSME_THRUST0);
		} else {
			pParent->SetThrusterIsp(thMe, SSME_ISP0 * 0.002, SSME_ISPSL * 0.002);
			pParent->SetThrusterMax0(thMe, SSME_THRUST0 * 0.002);
		}

		pParent->SetThrusterLevel(thMe, fThrustLevel);	
			
	}
}

bool SSME::CheckIgnitionConditions() const 
{
	if(IsRealized())
	{

		return ((pParent->GetThrusterLevel(thMe) > 0.3) && (IgniterFired() || IsIgnited() ));
	}
	else
		return false;
}

double SSME::GetChamberPressure() const
{
	if(IsRealized())
	{
		if(IsIgnited())
		{
			return pParent->GetThrusterLevel(thMe) * 199.0;
		} else 
		{
			return pParent->GetThrusterLevel(thMe) * 7.0;
		}
	}
	else {
		return 0.0;
	}
}

void SSME::DoEngineController(double fSimT, double fDeltaT)
{
	if(IsRealized())
	{
		switch(mode)
		{
		case MODE_IDLE:
			fIsGood = true;
			break;
		case MODE_START:
			fThrustCmd2 = 0.6;
			fIsGood = false;
			if(pParent->GetThrusterLevel(thMe) > 0.55) 
			{
				fTimer1 = 0.0;
				fTimer2 = 0.0;
				mode = MODE_IGNITION;
			}
			break;
		case MODE_IGNITION:
			if(!IsIgnited())
			{
				FireIgniter();
			} else {
				double fX = 1.5 * fTimer2;
				if(fX > 1.0)
					fX = 1.0;
				fThrustCmd2 = 0.55 + fX * 0.45;
			}
			fTimer2 += fDeltaT;
			if(GetChamberPressure() < 165.0) 
			{
				
				fIsGood = false;
				if(fTimer2 >= 1.6)
				{
					mode = MODE_SHUTDOWN;
				}
			} else {
				fTimer1 += fDeltaT;
				if(fTimer1 >= 1.6) {
					fIsGood = true;
					mode = MODE_RUNNING;
				}
			}
			break;
		case MODE_RUNNING:
			fThrustCmd2 = fThrustCmd;
			if(IsIgnited() && GetChamberPressure() >= 50.0)
			{
				fIsGood = true;
			} else {
				fIsGood = false;
			}
			if(!fIsGood)
			{
				mode = MODE_SHUTDOWN;
			}
			break;
		case MODE_SHUTDOWN:
			fThrustCmd2 = 0.0;
			fIsGood = true;
			break;
		}
	}
}


